#!/usr/bin/env python

"""
Handlers for a resource.
"""

from imiptools.content import Handler, format_datetime, get_address, get_uri, to_part
from imiptools.handlers.common import CommonFreebusy
from vCalendar import to_node

class Event(Handler):

    "An event handler."

    def add(self):
        pass

    def cancel(self):
        pass

    def counter(self):

        "Since this handler does not send requests, it will not handle replies."

        pass

    def declinecounter(self):

        """
        Since this handler does not send counter proposals, it will not handle
        replies to such proposals.
        """

        pass

    def publish(self):
        pass

    def refresh(self):
        pass

    def reply(self):

        "Since this handler does not send requests, it will not handle replies."

        pass

    def request(self):

        """
        Respond to a request by preparing a reply containing accept/decline
        information for each indicated attendee.

        No support for countering requests is implemented.
        """

        oa = self.require_organiser_and_attendees()
        if not oa:
            return None

        organiser_item, attendees = oa

        # Validate the organiser, ignoring spoofed requests.

        if not self.validate_identities([organiser_item]):
            return None

        # Process each attendee separately.

        calendar = []

        for attendee, attendee_attr in attendees.items():

            # Check for event using UID.

            if not self.have_new_object(attendee, "VEVENT"):
                continue

            # If newer than any old version, discard old details from the
            # free/busy record and check for suitability.

            periods = self.get_periods()
            freebusy = self.store.get_freebusy(attendee) or []
            scheduled = self.can_schedule(freebusy, periods)

            attendee_attr["PARTSTAT"] = scheduled and "ACCEPTED" or "DECLINED"
            if self.messenger and self.messenger.sender != get_address(attendee):
                attendee_attr["SENT-BY"] = get_uri(self.messenger.sender)
            self.details["ATTENDEE"] = [(attendee, attendee_attr)]

            event = to_node({"VEVENT" : [(self.details, {})]})
            calendar.append(event)
            self.store.set_event(attendee, self.uid, event)

            # Only update free/busy details if the event is scheduled.

            if scheduled:
                self.update_freebusy(freebusy, attendee, periods)
                if self.publisher:
                    self.publisher.set_freebusy(attendee, freebusy)

        return "REPLY", to_part("REPLY", calendar)

class Freebusy(Handler, CommonFreebusy):

    "A free/busy handler."

    def publish(self):
        pass

    def reply(self):

        "Since this handler does not send requests, it will not handle replies."

        pass

    # request provided by CommonFreeBusy.request

class Journal(Handler):

    "A journal entry handler."

    def add(self):
        pass

    def cancel(self):
        pass

    def publish(self):
        pass

class Todo(Handler):

    "A to-do item handler."

    def add(self):
        pass

    def cancel(self):
        pass

    def counter(self):

        "Since this handler does not send requests, it will not handle replies."

        pass

    def declinecounter(self):

        """
        Since this handler does not send counter proposals, it will not handle
        replies to such proposals.
        """

        pass

    def publish(self):
        pass

    def refresh(self):
        pass

    def reply(self):

        "Since this handler does not send requests, it will not handle replies."

        pass

    def request(self):
        pass

# Handler registry.

handlers = [
    ("VFREEBUSY",   Freebusy),
    ("VEVENT",      Event),
    ("VTODO",       Todo),
    ("VJOURNAL",    Journal),
    ]

# vim: tabstop=4 expandtab shiftwidth=4
